# type: string (compiler used for MPI build)
%{!?compiler: %define compiler gcc}
# type: string (device used for mpi build)
%{!?mpi_device: %define mpi_device ch_gen2}

# Set our root directory
%define mpidir %{_libdir}/%{name}/%{version}-%{compiler}

Summary: MPI implementation over Infiniband RDMA-enabled interconnect
Name: mvapich
Version: 1.2.0
Release: 0.3562.2%{?dist}
License: BSD
Group: Development/Libraries
Source0: mvapich-%{version}-3562.tar.gz
Source1: mvapich.sh.in
Source2: mvapich.csh.in
Patch0: mvapich-1.0.1-limit.patch
URL: http://mvapich.cse.ohio-state.edu/
Requires: mpi-selector, libibverbs-devel, libibumad-devel
Requires(post): mpi-selector, rpm
Requires(preun): mpi-selector, rpm
BuildRequires: libibverbs-devel >= 1.1.2-4.el5, libibumad-devel, perl, autoconf
ExclusiveArch: i386 x86_64 ia64 ppc64
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)

%if %(test "%{compiler}" != "gcc" && echo 1 || echo 0)
AutoReq: no
Requires: libibverbs libibumad
%else
BuildRequires: gcc-c++ gcc-gfortran
%endif

%description
This is high performance and scalable MPI-1 implementation over Infiniband
and RDMA-enabled interconnects.  This implementation is based on  MPICH
and MVICH. MVAPICH is pronounced as `em-vah-pich''. 

%prep
%setup -q -n %{name}-%{version}-3562
%patch0 -p1 -b .limit
OPTIMIZATION_FLAG="-O3 -fno-strict-aliasing"
CONFIG_ENABLE_F77="--enable-f77"
CONFIG_ENABLE_F90="--enable-f90"
EXTRA_CFLAG=
MPE_FLAGS=
conffile=mvapich.conf
buildidfile=BUILDID
#############################################################################
# Compiler definition
# GNU compilers
%if %(test "%{compiler}" = "gcc" && echo 1 || echo 0)
    export CC=gcc
    export CXX=g++
    gcc_ver=`gcc --version | head -1 | awk '{print$3}' | awk -F. '{print $1}'`
    # For systems with mixed gcc-fortran install.
    if [ $gcc_ver -gt 3 ]; then
        # new gcc version
        export FC=gfortran
        export F77=gfortran
        export F90=gfortran
        export F77_GETARGDECL=
    else
        # old gcc version
        export FC=g77
        export F77=g77
        export F90=g77
    fi
    export CFLAGS="-Wall"
    export FFLAGS="-fPIC"
    export CXXFLAGS="-fPIC"
    export F90FLAGS="-fPIC"
    export CONFIG_FLAGS=
    export COMPILER_CONFIG="--with-romio"
%endif
# Intel compiler
%if %(test "%{compiler}" = "intel" && echo 1 || echo 0)
    export CC=icc
    export CXX=icc
    export FC=ifort
    export F90=$FC
    export CFLAGS="-D__INTEL_COMPILER"
    export FFLAGS="-fPIC"
    export CXXFLAGS="-fPIC"
    export CCFLAGS="-lstdc++"
    export F90FLAGS=$FFLAGS
    export CONFIG_FLAGS=""
    export COMPILER_CONFIG="--enable-f90modules --with-romio"
%endif
# Pathscale compiler
%if %(test "%{compiler}" = "pathscale" && echo 1 || echo 0)
    export CC=pathcc
    export CXX=pathCC
    export FC=pathf90
    export F90=pathf90
    export F77=pathf90
    export CFLAGS=""
    export FFLAGS="-fPIC"
    export CXXFLAGS="-fPIC"
    export CCFLAGS=""
    export F90FLAGS=$FFLAGS
    export CONFIG_FLAGS=""
    export COMPILER_CONFIG="--enable-f90modules --with-romio"
%endif
# PGI compiler
%if %(test "%{compiler}" = "pgi" && echo 1 || echo 0)
    export CC=pgcc
    export CXX=pgCC
    export FC=pgf77
    export F90=pgf90
    export CFLAGS="-Msignextend -DPGI"
    export FFLAGS="-fPIC"
    export CXXFLAGS="-fPIC"
    export F90FLAGS=$FFLAGS
    export CONFIG_FLAGS=""
    export OPTIMIZATION_FLAG=""
%endif
# Sun Studio compiler
%if %(test "%{compiler}" = "sun" && echo 1 || echo 0)
    export CC=suncc
    export CXX=sunCC
    export F77=sunf77
    export F90=sunf90
    export CFLAGS=""
    export FFLAGS="-fPIC"
    export CXXFLAGS="-fPIC"
    export F90FLAGS=$FFLAGS
    export CONFIG_FLAGS=""
    export OPTIMIZATION_FLAG="-O3"
%endif

#############################################################################

EXTRA_CFLAG=$CFLAGS

# Customize your system architecture in ARCH_NAME
# "_IA32_" for i686, "_IA64_" for ia64, 
# "_X86_64_" for Opteron, "_EM64T_" for Intel EM64T
# "_PPC64_" for PowerPC 64
%ifarch i386
export ARCH="_IA32_"
if [ %{compiler} == "gcc" ]; then
    # Flag fixes for Fedora PPC
    CFLAGS="-m32 $CFLAGS"
    CXXFLAGS="-m32 $CXXFLAGS"
    CPPFLAGS="-m32 $CPPFLAGS"
    FFLAGS="-m32 $FFLAGS"
    F90FLAGS="-m32 $F90FLAGS"
    LDFLAGS="-m32 $LDFLAGS"
    USER_CFLAGS="-m32 $USER_CFLAGS"
    MPIRUN_CFLAGS="-m32 $MPIRUN_CFLAGS"
    MPE_FLAGS=-mpe_opts="--with-cflags=-m32 --with-fflags=-m32"
fi
%endif
%ifarch ia64
export ARCH="_IA64_"
if [[ (( -f /lib/ssa/libgcc_s.so ) || ( -f /usr/lib/libgcc_s.so )) ]]; then
    EXTRA_CFLAG=" -L/lib/ssa -L/usr/lib -lgcc_s"
fi
%endif
%ifarch ppc64
export ARCH="_PPC64_"
if [ %{compiler} == "gcc" ]; then
    # Flag fixes for Fedora PPC
    CFLAGS="-m64 $CFLAGS"
    CXXFLAGS="-m64 $CXXFLAGS"
    CPPFLAGS="-m64 $CPPFLAGS"
    FFLAGS="-m64 $FFLAGS"
    F90FLAGS="-m64 $F90FLAGS"
    LDFLAGS="-m64 $LDFLAGS"
    USER_CFLAGS="-m64 $USER_CFLAGS"
    MPIRUN_CFLAGS="-m64 $MPIRUN_CFLAGS"
    MPE_FLAGS=-mpe_opts="--with-cflags=-m64 --with-fflags=-m64"
fi
%endif
%ifarch x86_64
export ARCH="_X86_64_"
if [ %{compiler} == "gcc" ]; then
    # Flag fixes for Fedora PPC
    CFLAGS="-m64 $CFLAGS"
    CXXFLAGS="-m64 $CXXFLAGS"
    CPPFLAGS="-m64 $CPPFLAGS"
    FFLAGS="-m64 $FFLAGS"
    F90FLAGS="-m64 $F90FLAGS"
    LDFLAGS="-m64 $LDFLAGS"
    USER_CFLAGS="-m64 $USER_CFLAGS"
    MPIRUN_CFLAGS="-m64 $MPIRUN_CFLAGS"
    MPE_FLAGS=-mpe_opts="--with-cflags=-m64 --with-fflags=-m64"
fi
%endif

# check for version
if [ -f $buildidfile ]; then
    buildid=`cat $buildidfile | grep MVAPICH_BUILDID |awk '{print $2}'`
    if [ "$buildid" != "" ];then
        DEF_BUILDID="$DEF_BUILDID -DMVAPICH_BUILDID=\\\"$buildid\\\""
    else
        DEF_BUILDID=""
    fi
fi

# Must set these in order to keep configure from breaking
export IB_HOME=/usr
export IB_HOME_LIB=%{_libdir}
export PREFIX=%{mpidir}
export COMPAT=COMPAT_MODE
# end required settings
export RSHCOMMAND=ssh
export CFLAGS="$CFLAGS -D$ARCH $OPTIMIZATION_FLAG -g -D_GNU_SOURCE -DCH_GEN2 -D_AFFINITY_ -DMEMORY_SCALE -D_SMP_ -D_SMP_RNDV_ -DVIADEV_RPUT_SUPPORT -DEARLY_SEND_COMPLETION"
export USER_CFLAGS
export MPE_OPTS
export MPE_CFLAGS
export LDFLAGS
export CXXFLAGS="$CXXFLAGS"
export FFLAGS="$FFLAGS $EXTRA_CFLAG"
export F90FLAGS="$F90FLAGS $EXTRA_CFLAG"
export CONFIG_FLAGS
export MPIRUN_CFLAGS="$MPIRUN_CFLAGS -DPARAM_GLOBAL=\\\"%{mpidir}/etc/$conffile\\\" -DLD_LIBRARY_PATH_MPI=\\\"%{mpidir}/lib\\\" -DMPI_PREFIX=\\\"%{mpidir}/\\\" $DEF_BUILDID"
set -o pipefail
./configure --enable-sharedlib --with-device=%{mpi_device} --with-arch=LINUX -prefix=%{mpidir} --with-echo $CONFIG_ENABLE_F77 $CONFIG_ENABLE_F90 $COMPILER_CONFIG -lib="-libverbs -libumad -lpthread $EXTRA_CFLAG" $MPE_FLAGS $CONFIG_FLAGS

%build
# We found that the mvapich2 make system does smp unsafe things (at least when
# doing smp makes over NFS), so use a non-smp make here too just for safety
make

%install
rm -rf %{buildroot}
# We pass the final prefix to configure above, then override it here to get
# things installed in our build root
make DESTDIR=%{buildroot} install 

# Post install fixes
# Remove bad links
rm -f %{buildroot}%{mpidir}/share/examples/mpirun
# Remove the uninstall script
rm -f %{buildroot}%{mpidir}/sbin/mpiuninstall
# Change and enable some defaults in mvapich.conf
perl -pi -e "s,^# VIADEV_DEFAULT_MIN_RNR_TIMER=12,VIADEV_DEFAULT_MIN_RNR_TIMER=25,g" %{buildroot}%{mpidir}/etc/mvapich.conf

# mpi-selector scripts
sed 's#@MPIDIR@#'%{mpidir}'#g' < %SOURCE1 > %{buildroot}%{mpidir}/bin/mpivars.sh
sed 's#@MPIDIR@#'%{mpidir}'#g' < %SOURCE2 > %{buildroot}%{mpidir}/bin/mpivars.csh

%clean
rm -rf %{buildroot}

%post
# It is safe to reregister the same setup at every upgrade/reinstall/etc.
mpi-selector --register %{name}-%{version}-%{compiler}-%{_arch} \
    --source-dir %{mpidir}/bin --yes >/dev/null 2>&1
# We can't fail
/bin/true

%preun
# Count how many copies of this particular package are currently installed,
# knowing that on upgrade the new package being installed is already in this
# list by the time we are run, and that we are in the list too as we aren't
# removed until after our scripts complete.  If our count comes back as just
# 1, then we are the last copy of this particular version and we need to
# remove our registration.  This is true regardless of wheter this is an
# upgrade or an erase.  Note: we have to keep our arch in the query too or
# else multilib installations throw this count off.
COUNT=`rpm -q %{name}-%{version}.%{_arch} | wc -l`
if [ $COUNT -eq 1 ]; then
    mpi-selector --unregister %{name}-%{version}-%{compiler}-%{_arch} \
            --yes >/dev/null 2>&1
fi
/bin/true

%files
%defattr(-, root, root, -)
%dir %{mpidir}
%{mpidir}/*

%changelog
* Mon Aug 13 2012 Doug Ledford <dledford@redhat.com> - 1.2.0-0.3562.2.el5
- Bump and rebuild against latest libibverds, librdmacm, etc.
- Related: bz771441

* Tue Dec 22 2009 Doug Ledford <dledford@redhat.com> - 1.2.0-0.3562.1.el5
- Update to latest upstream version
- Related: bz518218

* Mon Jun 22 2009 Doug Ledford <dledford@redhat.com> - 1.1.0-0.3355.2.el5
- Rebuild against libibverbs that isn't missing the proper ppc wmb() macro
- Related: bz506258

* Sun Jun 21 2009 Doug Ledford <dledford@redhat.com> - 1.1.0-0.3355.1.el5
- Update to final ofed 1.4.1 bits
- Compile against non-XRC libibverbs
- Related: bz506258, bz506097

* Wed Apr 22 2009 Doug Ledford <dledford@redhat.com> - 1.1.0-0.3143.1
- Update to ofed 1.4.1-rc3 version
- Related: bz459652

* Thu Oct 16 2008 Doug Ledford <dledford@redhat.com> - 1.1.0-0.2931.3
- The mpivars files needed the shared library directory updated
- Resolves: bz466389

* Fri Oct 03 2008 Doug Ledford <dledford@redhat.com> - 1.1.0-0.2931.2
- Incorporate fixups to Requires(post) and Requires(preun) from openmpi
- Fix up post/preun scripts to work like the scripts from openmpi11
- Resolves: bz465448

* Thu Sep 18 2008 Doug Ledford <dledford@redhat.com> - 1.1.0-0.2931.1
- Rearrange the position of the svn tag in the n-v-r so z stream updates
  won't be so ugly should they ever be needed

* Wed Sep 17 2008 Doug Ledford <dledford@redhat.com> - 1.1.0-0.el5.2931.1
- Update to later upstream source, gut spec file and make it sane
- Resolves: bz451476

* Mon May  12 2008 Pavel Shamis <pasha@mellanox.co.il>
- Fixes for mvapich 1.1 and OFED 1.4:
* Mon May  12 2008 Pavel Shamis <pasha@mellanox.co.il>
- Adding SUN compiler support
- Adding -fPIC to Pathscale, GCC and Intel
* Sun May  4 2008 Pavel Shamis <pasha@mellanox.co.il>
- Removing unused code
* Mon Jan  7 2008 Pavel Shamis <pasha@mellanox.co.il>
- Enable F90 build on PPC platforms
* Thu Dec  6 2007 Pavel Shamis <pasha@mellanox.co.il>
- Adding -fPIC compilation flag for pgi compiler
* Mon Dec  5 2007 Pavel Shamis <pasha@mellanox.co.il>
- Removing explicit Provides
- Replacing autoreqprov autoreq
* Mon Dec  3 2007 Pavel Shamis <pasha@mellanox.co.il>
- Fixing PGI 7.1 failure
* Wed Oct 31 2007 Pavel Shamis <pasha@mellanox.co.il>
- Adding support for mvapich 1.0.0 version
- Fixing mpi-selector bug
* Wed Aug 12 2007 Pavel Shamis <pasha@mellanox.co.il>
- Replacing default mvapich tunings with new values.
* Wed Jun  6 2007 Pavel Shamis <pasha@mellanox.co.il>
- Fixed PGI build bug. PGI doesn't support -Wall flag.
* Sun Mar 25 2007 Pavel Shamis <pasha@mellanox.co.il>
- Added support for mpi_compat_mode. 
* Wed Mar  8 2007 Pavel Shamis <pasha@mellanox.co.il>
- Intel compiler name was changed from icc to intel
* Tue Mar  6 2007 Pavel Shamis <pasha@mellanox.co.il>
- Fixed bug in mvapich.sh/csh generation.
* Thu Mar  1 2007 Pavel Shamis <pasha@mellanox.co.il>
- OFED section was moved before rpm definition section.
* Thu Mar  1 2007 Pavel Shamis <pasha@mellanox.co.il>
- OFED section was moved before rpm definition section.
* Thu Feb 20 2007 Pavel Shamis <pasha@mellanox.co.il>
- added support for mpi-selector
* Mon Feb  5 2007 Pavel Shamis <pasha@mellanox.co.il>
- the %build macro was removed,workaround for SUSE issue - %build removes $RPM_BUILD_ROOT
- added "-libcommon" for SuSe10
* Tue Jan 30 2007 Pavel Shamis <pasha@mellanox.co.il>
- The spec file was created
